=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-07-04 15:04:11 +0000
+++ bzrlib/builtins.py	2006-07-11 20:45:32 +0000
@@ -2617,10 +2617,10 @@
     takes_args = ['location?']
     aliases = []
 
-    def run(self, location=None, 
+    def run(self, location=None,
             dry_run=False, verbose=False,
             revision=None, force=False):
-        from bzrlib.log import log_formatter
+        from bzrlib.log import log_formatter, show_log
         import sys
         from bzrlib.uncommit import uncommit
 
@@ -2634,18 +2634,33 @@
             tree = None
             b = control.open_branch()
 
+        rev_id = None
         if revision is None:
             revno = b.revno()
-            rev_id = b.last_revision()
         else:
-            revno, rev_id = revision[0].in_history(b)
+            # 'bzr uncommit -r 10' actually means uncommit
+            # so that the final tree is at revno 10.
+            # but bzrlib.uncommit.uncommit() actually uncommits
+            # the revisions that are supplied.
+            # So we need to offset it by one
+            revno = revision[0].in_history(b).revno+1
+
+        if revno <= b.revno():
+            rev_id = b.get_rev_id(revno)
         if rev_id is None:
-            print 'No revisions to uncommit.'
-
-        for r in range(revno, b.revno()+1):
-            rev_id = b.get_rev_id(r)
-            lf = log_formatter('short', to_file=sys.stdout,show_timezone='original')
-            lf.show(r, b.repository.get_revision(rev_id), None)
+            self.outf.write('No revisions to uncommit.\n')
+            return 1
+
+        lf = log_formatter('short',
+                           to_file=self.outf,
+                           show_timezone='original')
+
+        show_log(b,
+                 lf,
+                 verbose=False,
+                 direction='forward',
+                 start_revision=revno,
+                 end_revision=b.revno())
 
         if dry_run:
             print 'Dry-run, pretending to remove the above revisions.'

=== modified file 'bzrlib/tests/blackbox/test_uncommit.py'
--- bzrlib/tests/blackbox/test_uncommit.py	2006-07-11 20:27:57 +0000
+++ bzrlib/tests/blackbox/test_uncommit.py	2006-07-11 20:45:32 +0000
@@ -15,10 +15,10 @@
         wt = self.make_branch_and_tree('tree')
         self.build_tree(['tree/a', 'tree/b', 'tree/c'])
         wt.add(['a', 'b', 'c'])
-        wt.commit('initial', rev_id='a1')
+        wt.commit('initial commit', rev_id='a1')
 
         open('tree/a', 'wb').write('new contents of a\n')
-        wt.commit('second', rev_id='a2')
+        wt.commit('second commit', rev_id='a2')
 
         return wt
 
@@ -29,12 +29,16 @@
         os.chdir('tree')
         out, err = self.run_bzr('uncommit', '--dry-run', '--force')
         self.assertContainsRe(out, 'Dry-run')
+        self.assertNotContainsRe(out, 'initial commit')
+        self.assertContainsRe(out, 'second commit')
 
         # Nothing has changed
         self.assertEqual('a2', wt.last_revision())
 
         # Uncommit, don't prompt
-        self.run_bzr('uncommit', '--force')
+        out, err = self.run_bzr('uncommit', '--force')
+        self.assertNotContainsRe(out, 'initial commit')
+        self.assertContainsRe(out, 'second commit')
 
         # This should look like we are back in revno 1
         self.assertEqual('a1', wt.last_revision())
@@ -52,10 +56,14 @@
         os.chdir('checkout')
         out, err = self.run_bzr('uncommit', '--dry-run', '--force')
         self.assertContainsRe(out, 'Dry-run')
+        self.assertNotContainsRe(out, 'initial commit')
+        self.assertContainsRe(out, 'second commit')
 
         self.assertEqual('a2', checkout_tree.last_revision())
 
         out, err = self.run_bzr('uncommit', '--force')
+        self.assertNotContainsRe(out, 'initial commit')
+        self.assertContainsRe(out, 'second commit')
 
         # uncommit in a checkout should uncommit the parent branch
         # (but doesn't effect the other working tree)
@@ -83,3 +91,19 @@
         b.pull(t.branch)
         uncommit.uncommit(b)
 
+    def test_uncommit_revision(self):
+        wt = self.create_simple_tree()
+
+        os.chdir('tree')
+        out, err = self.run_bzr('uncommit', '-r1', '--force')
+
+        self.assertNotContainsRe(out, 'initial commit')
+        self.assertContainsRe(out, 'second commit')
+        self.assertEqual('a1', wt.last_revision())
+        self.assertEqual('a1', wt.branch.last_revision())
+
+    def test_uncommit_neg_1(self):
+        wt = self.create_simple_tree()
+        os.chdir('tree')
+        out, err = self.run_bzr('uncommit', '-r', '-1', retcode=1)
+        self.assertEqual('No revisions to uncommit.\n', out)

